--------------------------------------------------------------------------
-------------------------------------------------------------------------
--  CONSTRUO DA ARQUITETURA CLEPATRA COMPLETA
--
--  Fernando Moraes
--  Ney Calazans
--
--  Incio: 11/maio/1998             Ultima modificao:  17/agosto/2004
--
--  17/08/2004 - sincronizao do sinal externo hold na subida do sinal
--		externo de relgio e gerao da verso 3.0 de distribuio da
--  	arquitetura (Calazans)
--  02/08/2004 - mudana da Unidade de Controle para mquina de estados
--		(Calazans)
--  30/07/2004 - reviso para separar barramento bidirecional de dados
--		em dois barramentos unidirecionais e acrescentar um sinal de
--		hold na interface externa do processador. Estas so as primeiras
-- 		alteraes visando a prototipao em hardware. (Calazans)
--  28/07/2004 - reviso para modularizar implementao - (Calazans)
--  06/06/2003 - reviso do cdigo - (Calazans/Moraes)
--  04/08/2002 - insero do flag de overflow e decorrncias - (Calazans)
--  15/10/2001 - simplificao da Unidade de Controle e correo dos
--		jumps indiretos - (Moraes)
--
--  CPU: unio do bloco de dados com o bloco de controle
--------------------------------------------------------------------------
--------------------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use work.cleo.all;

entity cleopatra is
	port(clock, reset, hold	: in std_logic;
		 halt			: out std_logic;
         ce, rw			: out std_logic; 
         address		: out internal_bus;
         datain			: in internal_bus;
		 dataout		: out internal_bus
        ); 
end cleopatra;
    
architecture cleopatra of cleopatra is
   signal uins : microinstrucao; 
   signal n,z,c,v : std_logic;
   signal ir : internal_bus;
   SIGNAL hold_int : std_logic; -- hold interno
   
begin
   ce <= uins.ce;
   rw <= uins.rw;
   
   -- processo para registrar, e assim sincronizar, o sinal externo hold
   -- na borda de subida do relgio
   process (clock, reset)
   begin
	   if reset='1' then
		   hold_int <='0';   
	   elsif clock'event and clock='1' then
		   hold_int <= hold;
	   end if;
   end process;
	   
   DP: entity work.datapath port map (clock=>clock, reset=>reset,
	   hold=>hold_int, address=>address, datain=>datain, dataout=>dataout,
	   ir=>ir, uins=>uins, n=>n, z=>z, c=>c, v=>v);  
                           
   CU: entity work.control port map (clock=>clock, reset=>reset,
	   hold=>hold_int, halt=>halt, ir=>ir, uins=>uins,
	   n=>n, z=>z, c=>c, v=>v);
end cleopatra;
